package io.flutter.plugin.driver;

import android.text.TextUtils;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import com.suyun.app.lib.router.SYRouter;
import com.tencent.bugly.crashreport.CrashReport;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * DriverPlugin
 */
public class DriverPlugin implements FlutterPlugin, MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;

    private static final String CHANNEL_NAME = "com.suyun.app.io.flutter.plugin/driver";

    @Override
    public void onAttachedToEngine(FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), CHANNEL_NAME);
        channel.setMethodCallHandler(new DriverPlugin());
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-io.flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), CHANNEL_NAME);
        channel.setMethodCallHandler(new DriverPlugin());
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        if (call.method.equals("getHeaders")) {
            result.success(SYRouter.getInstance().autoLink("/net/headers"));
        } else if (call.method.equals("getParams")) {
            result.success(SYRouter.getInstance().autoLink("/net/params"));
        } else if (call.method.equals("report")) {
            //bugly 上报错误
            if (call.arguments instanceof HashMap) {
                report2Bugly(call);
            }
            result.success(true);
        } else {
            result.notImplemented();
        }
    }

    @Override
    public void onDetachedFromEngine(FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    private void report2Bugly(MethodCall call) {
        try {
            HashMap<String, String> arguments = (HashMap<String, String>) call.arguments;
            if (arguments.size() > 0) {
                CrashReport.postCatchedException(parseStack(arguments));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private Throwable parseStack(HashMap<String, String> arguments) {
        Throwable catchThrowable = null;
        try {
            List<StackTraceElement> stackTrace = new ArrayList<>();
            String trace = arguments.get("trace");
            String[] split = trace.split("#");
            for (int i = 0, size = split.length; i < size; i++) {
                String traceLine = split[i];
                if (TextUtils.isEmpty(traceLine)) {
                    continue;
                }
                if (i == 100) {
                    break;
                }
                String[] split1 = traceLine.split("\\.");
                String others = traceLine.substring(traceLine.indexOf(".") + 1);
                String[] split2 = others.split("\\(");


                String fileName = traceLine.substring(traceLine.indexOf("(") + 1, traceLine.lastIndexOf(":"));
                String methodName = split2[0];
                String className = split1[0];
                int lineNum = Integer.parseInt(traceLine.substring(traceLine.lastIndexOf(":") + 1, traceLine.lastIndexOf(")")));
                stackTrace.add(new StackTraceElement(className, methodName, fileName, lineNum));
            }
            catchThrowable = new Throwable(arguments.get("message"));

            StackTraceElement[] array = new StackTraceElement[stackTrace.size()];
            catchThrowable.setStackTrace(stackTrace.toArray(array));
        } catch (NumberFormatException e) {
            e.printStackTrace();
            catchThrowable = new Throwable(arguments.get("message") + "trace:" + arguments.get("trace"));
        }
        return catchThrowable;
    }
}
